<?php

class cpsmart_otp_woo
{
    public function __construct()
    {
        add_action('wp', [$this, 'add_checkout']);

        //Show status verify in admin orderDetail
        add_action('woocommerce_admin_order_data_after_billing_address', [$this, 'show_order_detail']);

        add_filter('manage_shop_order_posts_columns', [$this, 'columns_head'], 11, 1);
        add_action('manage_shop_order_posts_custom_column', [$this, 'columns_content'], 10, 2);

        add_action('woocommerce_thankyou', [$this, 'update_from_cookie'], 10, 1);
        add_action('woocommerce_product_options_downloads', [$this, 'admin_enable_verify'], 10, 1);
        add_action('save_post', [$this, 'save_product_meta'], 10, 1);

        add_action('wp_head', [$this, 'show_verify_to_download'], 10, 1);

        // AJAX
        add_action("wp_ajax_cpotp_download_file", [$this, 'handle_download']);
        add_action("wp_ajax_nopriv_cpotp_download_file", [$this, 'handle_download']);
    }

    public static function admin_get_action_page_after_checkout()
    {
        return apply_filters('actions_after_checkout', [
            '' => __cp("Don't show"),
            "woocommerce_thankyou" => __cp("After Billing address")
        ]);
    }

    public function show_verify_to_download()
    {
        if (is_singular('product')) {
            global $post;
            $product = wc_get_product($post->ID);
            if ($product->is_downloadable()) {
                $optionsVerify = cp_otp_page_setting::get_options();

                if (isset($optionsVerify['woo']['download_display']) && $optionsVerify['woo']['download_display'] > 0) {
                    $position = $optionsVerify['woo']['download_display'];
                } else {
                    $position = 15;
                }
                add_action('woocommerce_single_product_summary', [$this, 'replace_add_to_cart_button'], $position, 1);


                $cpotp_woo_download_sms = get_post_meta($product->get_id(), 'cpotp_woo_download_sms', true);
                $cpotp_woo_download_email = get_post_meta($product->get_id(), 'cpotp_woo_download_email', true);

                if ($cpotp_woo_download_sms != 'yes' && $cpotp_woo_download_email != 'yes') {
                    if (isset($optionsVerify['woo']['download_sms']) && $optionsVerify['woo']['download_sms']) {
                        $cpotp_woo_download_sms = $optionsVerify['woo']['download_sms'];
                    }

                    if (isset($optionsVerify['woo']['download_email']) && $optionsVerify['woo']['download_email']) {
                        $cpotp_woo_download_email = $optionsVerify['woo']['download_email'];
                    }
                }

                if ($cpotp_woo_download_sms == 'yes' || $cpotp_woo_download_email == 'yes') {
                    if (isset($optionsVerify['woo']['download_hide_add_cart']) && $optionsVerify['woo']['download_hide_add_cart']) {
                        remove_action('woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart', 30);
                    }
                }
            }
        }
    }


    public function handle_download()
    {
        global $wpdb;
        $id = \CastPlugin\CpCore::getRequest('id', ['strip_tags', 'trim']);
        $hash = \CastPlugin\CpCore::getRequest('cpotp_id', ['strip_tags', 'trim']);
        $product_id = \CastPlugin\CpCore::getRequest('product_id', ['strip_tags', 'trim']);

        $cpotp_phone = isset($_COOKIE['cpotp_phone']) ? $_COOKIE['cpotp_phone'] : '';
        $cpotp_email = isset($_COOKIE['cpotp_email']) ? $_COOKIE['cpotp_email'] : '';

        if (empty($cpotp_email) && empty($cpotp_phone)) {
            cpsmart_otp_templates::page_error(__cp("Access deny"));
        }

        if (empty($id)) {
            cpsmart_otp_templates::page_error(__cp("Result not found"));
        }

        $product = wc_get_product($product_id);

        $file_path = false;
        $dataItem = [];
        foreach ($product->get_downloads() as $downloadItem) {
            if ($downloadItem['id'] == $id) {
                $file_path = $downloadItem['file'];
                $dataItem = [
                    'id' => $downloadItem['id'],
                    'name' => $downloadItem['name'],
                    'file' => $downloadItem['file'],
                ];
            }
        }

        if (!$file_path) {
            cpsmart_otp_templates::page_error(__cp("File not found"));
        }

        $user = false;
        if (username_exists($cpotp_phone)) {
            $user = get_user_by('login', $cpotp_phone);
        } elseif (email_exists($cpotp_email)) {
            $user = get_user_by('email', $cpotp_email);
        }

        if ($user) {
            $cpotp_email = get_user_meta($user->ID, 'cpotp_email', true);
            $cpotp_phone = get_user_meta($user->ID, 'cpotp_phone', true);
        }

        $record = cpsmart_otp_log::getDownLoadRow($product_id, $id, $cpotp_phone, $cpotp_email);

        if (!$record) {
            $datasave = [
                "token" => $id,
                "files" => $file_path,
                "object_id" => $product_id,
                "object_type" => "woo_product",
                "phone" => $cpotp_phone,
                "email" => $cpotp_email,
                "user_id" => ($user ? $user->ID : ''),
                "limit" => 0,
                "created_at" => time(),
                "updated_at" => time(),
                "count" => 1,
                "status" => cpsmart_otp_log::DOWNLOAD_ACTIVE,
            ];

            $wpdb->show_errors(true);
            $wpdb->insert(cpsmart_otp_log::get_download_table(), $datasave);
        } else {
            $wpdb->update(cpsmart_otp_log::get_download_table(), [
                'count' => $record->count + 1,
                'files' => $file_path,
                "phone" => $cpotp_phone,
                "email" => $cpotp_email,
                "user_id" => ($user ? $user->ID : ''),
            ], [
                'id' => $record->id
            ]);
        }


        $filename = basename($file_path);

        if (strstr($filename, '?')) {
            $filename = current(explode('?', $filename));
        }

        $filename = apply_filters('woocommerce_file_download_filename', $filename, '');
        $file_download_method = "force";

        // Add action to prevent issues in IE.
        add_action('nocache_headers', array(__CLASS__, 'ie_nocache_headers_fix'));

        // Trigger download via one of the methods.
        do_action('woocommerce_download_file_' . $file_download_method, $file_path, $filename);
    }


    public function replace_add_to_cart_button()
    {
        global $product;

        if ($product->is_downloadable()) {
            $cpotp_phone = isset($_COOKIE['cpotp_phone']) ? $_COOKIE['cpotp_phone'] : '';
            $cpotp_email = isset($_COOKIE['cpotp_email']) ? $_COOKIE['cpotp_email'] : '';

            $cpotp_woo_download_sms_text = get_post_meta($product->get_id(), 'cpotp_woo_download_sms_text', true);
            $cpotp_woo_download_email_text = get_post_meta($product->get_id(), 'cpotp_woo_download_email_text', true);
            $cpotp_woo_download_sms = get_post_meta($product->get_id(), 'cpotp_woo_download_sms', true);
            $cpotp_woo_download_email = get_post_meta($product->get_id(), 'cpotp_woo_download_email', true);

            $show_button = true;

            if ($cpotp_woo_download_sms != 'yes' && $cpotp_woo_download_email != 'yes') {
                $optionsVerify = cp_otp_page_setting::get_options();

                if (isset($optionsVerify['woo']['download_sms']) && $optionsVerify['woo']['download_sms']) {
                    $cpotp_woo_download_sms = $optionsVerify['woo']['download_sms'];
                }

                if (isset($optionsVerify['woo']['download_email']) && $optionsVerify['woo']['download_email']) {
                    $cpotp_woo_download_email = $optionsVerify['woo']['download_email'];
                }

                $cpotp_woo_download_sms_text = empty($cpotp_woo_download_sms_text) ? $optionsVerify['woo']['download_sms_btn_text'] : $cpotp_woo_download_sms_text;
                $cpotp_woo_download_email_text = empty($cpotp_woo_download_email_text) ? $optionsVerify['woo']['download_email_btn_text'] : $cpotp_woo_download_email_text;
            }

            $cpotp_woo_download_sms_text = empty($cpotp_woo_download_sms_text) ? __cp("Download via Phone") : $cpotp_woo_download_sms_text;
            $cpotp_woo_download_email_text = empty($cpotp_woo_download_email_text) ? __cp("Download via Email") : $cpotp_woo_download_email_text;


            if ($cpotp_woo_download_sms == 'yes' || $cpotp_woo_download_email == 'yes') {
                wp_enqueue_script('fb-account-kit');
                wp_enqueue_script('cpotp-process-woo-download');

                echo "<div class='cpotp-download-area'>";


                if (!empty($cpotp_email) || !empty($cpotp_phone)) {
                    $show_button = false;
                    echo cpsmart_otp_templates::get_table_download_woocommerce($product);
                }
            }

            if ($show_button) {
                if ($cpotp_woo_download_sms == 'yes') {
                    $attr = [];
                    $attr['class'] = 'button cpotp-woo-download cpotp-sms';
                    $attr['onclick'] = 'CpOtp.fblogin_sms(event, "woo_download", "' . $product->get_id() . '")';

                    $attr = cpsmart_otp_templates::format_atts($attr);

                    echo sprintf('<button %1$s >%2$s</button>', $attr, $cpotp_woo_download_sms_text);
                }

                if ($cpotp_woo_download_email == 'yes') {
                    $attr = [];
                    $attr['class'] = 'button cpotp-woo-download cpotp-email';
                    $attr['onclick'] = 'CpOtp.fblogin_email(event, "woo_download", "' . $product->get_id() . '")';

                    $attr = cpsmart_otp_templates::format_atts($attr);

                    echo sprintf('<button %1$s >%2$s</button>', $attr, $cpotp_woo_download_email_text);
                }
            }


            if ($cpotp_woo_download_sms == 'yes' || $cpotp_woo_download_email == 'yes') {
                echo "</div>";
            }
        }
    }

    public function save_product_meta($post_id)
    {
        $post = get_post($post_id);
        if ($post->post_type == 'product') {
            if (isset($_POST['cpotp_woo_download_sms'])) {
                $cpotp_woo_download_sms = sanitize_text_field($_POST['cpotp_woo_download_sms']);
            } else {
                $cpotp_woo_download_sms = "no";
            }

            if (isset($_POST['cpotp_woo_download_email'])) {
                $cpotp_woo_download_email = sanitize_text_field($_POST['cpotp_woo_download_email']);
            } else {
                $cpotp_woo_download_email = "no";
            }

            $cpotp_woo_download_sms_text = isset($_POST['cpotp_woo_download_sms_text']) ? $_POST['cpotp_woo_download_sms_text'] : '';
            $cpotp_woo_download_email_text = isset($_POST['cpotp_woo_download_email_text']) ? $_POST['cpotp_woo_download_email_text'] : '';

            update_post_meta($post_id, "cpotp_woo_download_sms", $cpotp_woo_download_sms);
            update_post_meta($post_id, "cpotp_woo_download_email", $cpotp_woo_download_email);
            update_post_meta($post_id, "cpotp_woo_download_sms_text", $cpotp_woo_download_sms_text);
            update_post_meta($post_id, "cpotp_woo_download_email_text", $cpotp_woo_download_email_text);
        }
    }


    public function admin_enable_verify()
    {
        global $post, $thepostid, $product_object;

        $optionsVerify = cp_otp_page_setting::get_options();

        $cpotp_woo_download_sms = get_post_meta($thepostid, 'cpotp_woo_download_sms', true);
        $cpotp_woo_download_sms_text = get_post_meta($thepostid, 'cpotp_woo_download_sms_text', true);
        $cpotp_woo_download_email = get_post_meta($thepostid, 'cpotp_woo_download_email', true);
        $cpotp_woo_download_email_text = get_post_meta($thepostid, 'cpotp_woo_download_email_text', true);

        if ($cpotp_woo_download_sms == '') {
            $cpotp_woo_download_sms  = $optionsVerify['woo']['download_sms'];
            if ($cpotp_woo_download_sms) {
                $cpotp_woo_download_sms = "yes";
            }
        }
        if ($cpotp_woo_download_email == '') {
            $cpotp_woo_download_email  = $optionsVerify['woo']['download_email'];
            if ($cpotp_woo_download_email) {
                $cpotp_woo_download_email = "yes";
            }
        }

        $cpotp_woo_download_sms = $cpotp_woo_download_sms == 'yes' ? $cpotp_woo_download_sms : 'no';
        $cpotp_woo_download_email = $cpotp_woo_download_email == 'yes' ? $cpotp_woo_download_email : 'no';

        $cpotp_woo_download_sms_text = empty($cpotp_woo_download_sms_text) ? __cp("Download via Phone") : $cpotp_woo_download_sms_text;
        $cpotp_woo_download_email_text = empty($cpotp_woo_download_email_text) ? __cp("Download via Email") : $cpotp_woo_download_email_text;

        woocommerce_wp_checkbox(
            array(
                'id' => 'cpotp_woo_download_sms',
                'value' => $cpotp_woo_download_sms,
                'wrapper_class' => 'show_if_simple show_if_variable',
                'label' => __cp("Download with SMS"),
                'description' => __cp("Check to enable"),
            )
        );
        woocommerce_wp_text_input(
            array(
                'id' => 'cpotp_woo_download_sms_text',
                'value' => $cpotp_woo_download_sms_text,
                'label' => __cp("Text button"),
                'description' => "",
                'wrapper_class' => 'show_if_cpotp_woo_download_sms show_if_simple show_if_variable',
            )
        );

        woocommerce_wp_checkbox(array(
            'id' => 'cpotp_woo_download_email',
            'value' => $cpotp_woo_download_email,
            'wrapper_class' => 'show_if_simple show_if_variable',
            'label' => __cp('Download with Email'),
            'description' => __cp("Check to enable"),
        ));

        woocommerce_wp_text_input(
            array(
                'id' => 'cpotp_woo_download_email_text',
                'value' => $cpotp_woo_download_email_text,
                'label' => __cp("Text button"),
                'description' => "",
                'wrapper_class' => 'show_if_cpotp_woo_download_email show_if_simple show_if_variable',
            )
        );
    }

    public function update_from_cookie($order_id)
    {
        $cpotp_phone = isset($_COOKIE['cpotp_phone']) ? $_COOKIE['cpotp_phone'] : '';
        $cpotp_email = isset($_COOKIE['cpotp_email']) ? $_COOKIE['cpotp_email'] : '';
        $order = wc_get_order($order_id);
        $_GET['referral'] = $order->get_checkout_order_received_url();

        if (!empty($cpotp_email)) {
            $this->update_order_by_email($cpotp_email, $order_id, 'woocommerce_thankyou');
        } elseif (!empty($cpotp_phone)) {
            $this->update_order_by_phone($cpotp_phone, $order_id, 'woocommerce_thankyou');
        }
    }

    public function columns_head($defaults)
    {
        $defaults['cpotp_verify'] = __cp('Smart OTP Verified');

        return $defaults;
    }

    public function columns_content($column_name, $order_id)
    {
        if ($column_name == 'cpotp_verify') {
            $status_verify = get_post_meta($order_id, 'cpotp_verify_type', true);
            $data_verify = get_post_meta($order_id, 'cpotp_verify_data', true);

            if (in_array($status_verify, ['email', 'phone'])) {
                echo "<p><span class=\"cpotp-status cpotp-verified\"><span>{$data_verify}</span></span></p>";
            }
        }
    }

    public function show_order_detail($order)
    {
        $order_id = $order->get_id();

        $status_verify = get_post_meta($order_id, 'cpotp_verify_type', true);
        $data_verify = get_post_meta($order_id, 'cpotp_verify_data', true);

        $status_verify = esc_html($status_verify);
        $data_verify = esc_html($data_verify);

        if (in_array($status_verify, ['email', 'phone'])) {
            $text = sprintf(__cp("Verified by %s:"), $status_verify);
            echo "<p><b>{$text}</b></p>";
            echo "<p><span class=\"order-status status-on-hold\"><span>{$data_verify}</span></span></p>";
        }
    }


    public function add_checkout()
    {
        if (is_order_received_page()) {
            $optionsVerify = cp_otp_page_setting::get_options();
            if ($optionsVerify['woo']['enable'] == true) {
//                wp_send_json($optionsVerify);
                if ($optionsVerify['woo']['thankyou_page'] && !empty($optionsVerify['woo']['thankyou_page_action'])) {
                    add_filter('cpotp-js-variable', [$this, 'change_variable_js']);
                    add_action($optionsVerify['woo']['thankyou_page_action'], [$this, 'show_page_checkout_by_action']);
                }
            }
        }
    }

    public function change_variable_js($args)
    {
        global $wp;

        $args['woo'] = true;
        $order_id = $wp->query_vars['order-received'];

        $order_id = apply_filters('woocommerce_thankyou_order_id', absint($order_id));
        $order_key = apply_filters('woocommerce_thankyou_order_key', empty($_GET['key']) ? '' : wc_clean(wp_unslash($_GET['key']))); // WPCS: input var ok, CSRF ok.

        $order = false;
        if ($order_id > 0) {
            $order = wc_get_order($order_id);
            if (!$order || $order->get_order_key() !== $order_key) {
                $order = false;
            }
        }

        if (!$order) {
            return $args;
        }

        $args['order'] = [
            'phone' => $order->get_billing_phone(),
            'email' => $order->get_billing_email(),
            'order_id' => $order->get_id(),
            'form_name' => 'woocommerce_thankyou'
        ];

        return $args;
    }


    public function show_page_checkout_by_action($order_id)
    {
        wp_enqueue_script('fb-account-kit');
        wp_enqueue_script('cpotp-process-woo');

        $order = wc_get_order($order_id);

        $cp_otp_options = cp_otp_page_setting::get_options();

        $status_verify = get_post_meta($order_id, 'cpotp_verify_type', true);
        $data_verify = get_post_meta($order_id, 'cpotp_verify_data', true);

        $allow_login_email = $allow_login_sms = false;
        if (isset($cp_otp_options['woo']['mode_sms']) && $cp_otp_options['woo']['mode_sms'] == true) {
            $allow_login_sms = true;
        }
        if (isset($cp_otp_options['woo']['mode_email']) && $cp_otp_options['woo']['mode_email'] == true) {
            $allow_login_email = true;
        }


        include CPOTP_PATH . '/public/partials/cp-form-woo.php';
    }

    public static function update_order_by_phone($cpotp_phone, $order_id, $writeLog = false)
    {
        if (get_post_meta($order_id, 'cpotp_verify_data', true) == '') {
            update_post_meta($order_id, 'cpotp_verify_type', 'phone');
            update_post_meta($order_id, 'cpotp_verify_data', $cpotp_phone);
            self::update_status_affter_verify($order_id, $cpotp_phone);

            if ($writeLog) {
                $prefix_phone = isset($_COOKIE['cpotp_phone_prefix']) ? $_COOKIE['cpotp_phone_prefix'] : '';
                $referral = isset($_GET['referral']) ? esc_url($_GET['referral']) : "";
                cpsmart_otp_log::add_log_woo([
                    'phone' => $cpotp_phone,
                    'phone_prefix' => $prefix_phone,
                    'order_id' => $order_id,
                    'user_id' => "",
                    'email' => "",
                    'form_name' => $writeLog,
                    'referral' => $referral
                ]);
            }
        }
    }

    public static function update_order_by_email($cpotp_email, $order_id, $writeLog = false)
    {
        if (get_post_meta($order_id, 'cpotp_verify_data', true) == '') {
            update_post_meta($order_id, 'cpotp_verify_type', 'email');
            update_post_meta($order_id, 'cpotp_verify_data', $cpotp_email);
            self::update_status_affter_verify($order_id, $cpotp_email);

            if ($writeLog) {
                cpsmart_otp_log::add_log_woo([
                    'phone' => "",
                    'phone_prefix' => "",
                    'order_id' => $order_id,
                    'user_id' => "",
                    'email' => $cpotp_email,
                    'form_name' => $writeLog,
                    'referral' => isset($_GET['referral']) ? esc_url($_GET['referral']) : "",
                ]);
            }
        }
    }

    public static function update_status_affter_verify($order_id, $data)
    {
        $cp_otp_options = cp_otp_page_setting::get_options();
        $orderObject = wc_get_order($order_id);
        $new_status = apply_filters('cpotp_status_order_affer_verify', $cp_otp_options['woo']['status_verify']);
        if (!empty($new_status) && $new_status != 'none') {
            $orderModel = new WC_Order($orderObject->get_id());
            $orderModel->update_status($new_status, __cp("Update status after verified by {$data}"));
        }
    }
}

add_action('plugins_loaded', 'cp_otp_init_woo');
function cp_otp_init_woo()
{
    if (class_exists('WooCommerce')) {
        new cpsmart_otp_woo();
    }
}
